/*
Block Quest:
Work in progress!!

... Knights of the Square :)
*/

; --------------------------------------------------------------------

; creates 3 x 16 KB slots for ROM and 1 x 8 KB slot for RAM
.memorymap
defaultslot 2
slotsize $4000
slot 0 $0000
slot 1 $4000
slot 2 $8000
slotsize $2000
slot 3 $C000
.endme

; maps 32 KB of ROM into 2 x 16 KB banks
.rombankmap
bankstotal 2
banksize $4000
banks 2
.endro

; include the bluelib library assumed to be in the same directory
.INCLUDE "bluelib.inc"

; for bit-testing the borders of cells
.define NORTH      0
.define EAST       1
.define SOUTH      2
.define WEST       3

; colors for patterns
.define YELLOW     $2f
.define GREY       $15
.define MAROON     $01
.define SIENNA     $16
.define CRIMSON    $02
.define SANDY      $0b
.define GREEN      $0c
.define BLACK      $00
.define MARINE     $10

; positions in the SAT
.define PLRSAT     0               ; SAT index of the player sprite

; --------------------------------------------------------------------
.ramsection "Variables" slot 3
plrCell      db                    ; the cell of player's hot spot
plrCol       db                    ; he horisontal pixel offset
plrRow       db                    ; the vertical pixel offset
plrX         db                    ; horiz. pos. on the screen (0-256)
plrY         db                    ; vert. pos. on the screen (0-192)
plrAnim      db                    ; cel of animation
plrCC        db                    ; character code of player sprite

lvWidth      db                    ; width of current level
lvHeight     db                    ; height of current level
lvCell       dw                    ; cell table of current level
.ends
; --------------------------------------------------------------------

.bank 0 slot 0
.org 0
.section "Startup" force
             di
             im    1
             ld    sp, $dff0       ; stack pointer near end of RAM
             jp    main
.ends

.section "Main" free
main:        call  initBlib        ; initialize bluelib

; load assets for level 1: Village on Fire
             ld    hl, firePal
             call  setCRAM         ; define all colors in CRAM

             ld    hl, fireTile
             ld    bc, 18*32
             ld    de, $0000
             call  setVRAM         ; load tiles into tilebank

             ld    hl, fireScrn
             ld    bc, 768*2
             ld    de, $3800
             call  setVRAM         ; define screen map

; initialize key variables for level 1
             ld    hl, fireCell    ; pass pointer to cell attr. tabl.
             ld    de, fireMap     ; pass pointer to level attributes
             call  lvVars          ; level attributes into variables

; put a standing Arthur on the screen:
             ld    hl, animWalk    ; get charcode from cel array
             ld    a, 0
             ld    (plrAnim), a    ; initialize cel index
             call  arrayItm
             ld    c, a            ; C = charcode
             ld    a, (plrX)
             ld    d, a            ; D = VPos
             ld    a, (plrY)
             ld    e, a            ; E = HPos
             ld    b, PLRSAT       ; B = Sprite index in SAT
             call  goSprite

             ld     a, DSPON       ; turn display on
             call   toglDSP
             ei

; main game loop for level 1 so far...
gameLoop:    call  getPlr1         ; get input indirectly via bluelib
             ld    ix, plrCell
             ld    hl, fireCell
             ld    de, fireMap
             bit   CTDOWN, a       ; directional button down?
             call  nz, mvSouth

             call  getPlr1         ; get input indirectly via bluelib
             ld    ix, plrCell
             ld    hl, fireCell
             ld    de, fireMap
             bit   CTUP, a         ; directional button up?
             call  nz, mvNorth

             call  getPlr1         ; get input indirectly via bluelib
             ld    ix, plrCell
             ld    hl, fireCell
             ld    de, fireMap
             bit   CTRIGHT, a      ; directional button right?
             call  nz, mvEast

             call  getPlr1         ; get input indirectly via bluelib
             ld    ix, plrCell
             ld    hl, fireCell
             ld    de, fireMap
             bit   CTLEFT, a      ; directional button right?
             call  nz, mvWest

; handle player animation and load sprite pos and charcode to buffer
             call  getPlr1
             ld    b, %00001111    ; mask: directional buttons
             and   b
             jp    nz, plrWalk

plrStand:    jp    finLoop         ; don't animate if no directional
                                   ; button input
plrWalk:     ld    hl, plrAnim
             ld    de, animWalk
             call  advcAnim        ; advance walking anim by 1 cel

             ld    hl, animWalk    ; get charcode from cel array
             ld    a, (plrAnim)
             call  arrayItm
             ld    c, a            ; C = charcode
             ld    a, (plrX)
             ld    d, a            ; D = VPos
             ld    a, (plrY)
             ld    e, a            ; E = HPos
             ld    b, PLRSAT       ; B = Sprite index in SAT
             call  goSprite
             jp    finLoop

finLoop:     halt                  ; finish loop by waiting for ints.
             halt                  ; = this game runs at 30 FPS?
             jp    gameLoop
.ends

.orga $0038
.section "Maskable interrupt handler" force
             call  hdlFrame
             ei
             reti
.ends

.orga $0066
.section "Non-Maskable interrupt handler (pause)" force
             retn                  ; disable pause button
.ends

.section "Functions for handling collision detection and movement" free
; 4 similar functions for handling collision detection* and possible
; movement in 4 directions for a given entity on a given map/level
; Entry: ix = cell, col, row
; TODO: must make this section much leaner... but how?!
; move the entity 1 step to the south, if it is possible
;* collision detection between player object and background
mvSouth:     ld    a, 0            ; is entity at top row in the cell?
             cp    (ix + 2)        ; compare entity row to 0
             jp    nz, incRow      ; if not 0, then go ahead and move

             ld    a, (ix + 0)     ; check southern cell border
             ld    d, SOUTH
             call  chkBordr        ; is it passable?
             ret    nz             ; if not, then abort move

             ld    a, 0            ; overlapping cell to the right?
             cp    (ix + 1)        ; compare 0 to entity's col
             jp    z, incRow       ; if 0, then no overlap = go move

             ld    a, (ix + 0)     ; check southern border on the
             inc   a               ; cell to the right
             ld    d, SOUTH
             call  chkBordr
             ret    nz             ; abort if border is no pass

incRow:      inc (ix + 2)          ; add 1 to entity's row
             ld    a, 8            ; are we moving into a new cell?
             cp    (ix + 2)
             jp     nz, +
             ld    (ix + 2), $00   ; if so, reset row offset

             ld    a, (lvWidth)    ; get width of current level
             add   a, (ix + 0)
             ld    (ix + 0), a     ; add it to current cell
+:
             inc   (ix + 4)        ; update the entity's screen pos.
             ret

mvNorth:     ld    a, 0            ; is entity at top row in the cell?
             cp    (ix + 2)        ; compare entity row to 0
             jp    nz, decRow      ; if not 0, then go ahead and move

             ld    a, (ix + 0)     ; check northern cell border
             ld    d, NORTH
             call  chkBordr        ; is it passable?
             ret    nz             ; if not, then abort move

             ld    a, 0            ; overlapping cell to the right?
             cp    (ix + 1)        ; compare 0 to entity's col
             jp    z, decRow       ; if 0, then no overlap = go move

             ld    a, (ix + 0)     ; check northern border on the
             inc   a               ; cell to the right
             ld    d, NORTH
             call  chkBordr
             ret    nz             ; abort if border is no pass

decRow:      dec (ix + 2)          ; sub 1 from entity's row
             ld    a, $ff          ; are we moving into a new cell?
             cp    (ix + 2)
             jp     nz, +
             ld    (ix + 2), $07   ; if so, enter new cell at bottom

             ld    hl, lvWidth    ; get width of current level
             ld    a, (ix + 0)
             sub   (hl)
             ld    (ix + 0), a     ; subtract it from current cell
+:
             dec   (ix + 4)        ; update the entity's screen pos.
             ret

mvEast:      ld    a, 0            ; is entity at right col in the cell?
             cp    (ix + 1)        ; compare entity col to 0
             jp    nz, incCol      ; if not 0, then go ahead and move

             ld    a, (ix + 0)     ; check eastern cell border
             ld    d, EAST
             call  chkBordr        ; is it passable?
             ret    nz             ; if not, then abort move

             ld    a, 0            ; overlapping cell below?
             cp    (ix + 2)        ; compare 0 to entity's row
             jp    z, incCol       ; if 0, then no overlap = go move

             ld    a, (ix + 0)     ; check eastern border on the
             ld    hl, lvWidth     ; cell below
             add   a, (hl)
             ld    d, EAST
             call  chkBordr
             ret    nz             ; abort if border is no pass

incCol:      inc (ix + 1)          ; add 1 to entity's col
             ld    a, 8            ; are we moving into a new cell?
             cp    (ix + 1)
             jp     nz, +
             ld    (ix + 1), $00   ; if so, reset row offset
             inc   (ix + 0)        ; inc cell number
+:
             inc   (ix + 3)        ; update the entity's screen pos.
             ret

mvWest:      ld    a, 0            ; is entity at right col in the cell?
             cp    (ix + 1)        ; compare entity col to 0
             jp    nz, decCol      ; if not 0, then go ahead and move

             ld    a, (ix + 0)     ; check western cell border
             ld    d, WEST
             call  chkBordr        ; is it passable?
             ret    nz             ; if not, then abort move

             ld    a, 0            ; overlapping cell below?
             cp    (ix + 2)        ; compare 0 to entity's row
             jp    z, decCol       ; if 0, then no overlap = go move

             ld    a, (ix + 0)     ; check western border on the
             ld    hl, lvWidth     ; cell below
             add   a, (hl)
             ld    d, WEST
             call  chkBordr
             ret    nz             ; abort if border is no pass

decCol:      dec   (ix + 1)        ; sub 1 from entity's col
             ld    a, $ff          ; are we moving into a new cell?
             cp    (ix + 1)
             jp    nz, +
             ld    (ix + 1), $07   ; if so, place along right border
             dec   (ix + 0)        ; dec cell number
+:
             dec   (ix + 3)        ; update the entity's screen pos.
             ret
.ends

.section "Misc. functions" free
; -------------------------------------------------------------------
; LOAD LEVEL ATTRIBUTES INTO VARIABLES
; entry: HL = pointer to cell attrib. table
;        DE = pointer to map/level data
lvVars:      ld     a, h           ; load pointer to cell attrib. tbl
             ld     (lvCell), a
             ld     a, l
             ld     (lvCell + 1), a

             ex     de, hl
             ld     a, (hl)        ; lvl width and height (in cells)
             ld     (lvWidth), a
             inc    hl
             ld     a, (hl)
             ld     (lvHeight), a

             inc    hl
             ld     a, (hl)
             ld    (plrCell), a    ; the cell where the player starts
             inc    hl
             ld     a, (hl)
             ld    (plrRow), a     ; the starting row
             inc    hl
             ld     a, (hl)
             ld    (plrCol), a     ; the starting column
             inc    hl
             ld     a, (hl)
             ld     (plrX), a      ; sprite's initial HPos on screen
             inc    hl
             ld     a, (hl)
             ld     (plrY), a      ; initial VPos on screen

             ret
; -------------------------------------------------------------------
; LEVEL CELL ATTRIBUTES TABLE TO HL
; get 16 bit level cell table pointer from RAM and return it in HL
lvCellHL:    push  af
             ld    a, (lvCell)
             ld    h, a
             ld    a, (lvCell + 1)
             ld    l, a
             pop   af
             ret
; -------------------------------------------------------------------
; ADVANCE ANIMATION
; advance animation one cel
; HL = variable containing cel index, DE = pointer to animation data
advcAnim:    push  hl
             ld    a, (hl)         ; forward to next cel in animation
             ld    b, a
             inc   b
             ld    h, 0
             ld    l, b
             ld    de, animWalk
             add   hl, de
             ld    a, (hl)
             cp    $ff             ; if we meet the terminator $ff
             jp    nz, +           ; reset animation to cel 0
             ld    b, 0
+:           ld    a, b
             pop   hl
             ld    (hl), a
             ret
; -------------------------------------------------------------------
; GET ARRAY ITEM
; get byte sized item from array
; Entry: HL = array, A = index (8 bit index)
; Exit: A = item at given position
arrayItm:    ld    d, 0
             ld    e, a
             add   hl, de
             ld    a, (hl)
             ret
; -------------------------------------------------------------------
; CHECK BORDER
; test the border of a cell to see if it is passable
; parameters: A = cell index of the cell to test
;             D = border to test (north, east, south, west)
; sets the zero flag if the specified border is passable
chkBordr:    call  lvCellHL
             ld    b, $00
             ld    c, a
             add   hl, bc          ; hl pointing to cell in table

             ld    a, d
             cp    NORTH           ; shall we test northern border?
             jp    nz, +           ; if not, >> to next direction
             ld    a,(hl)          ; read the cell's attributes
             bit   0, a            ; test bit 0 = northern border 0/1
             jp    finTest         ; zero flag is set, now return...
+:           cp    EAST            ; test eastern border...?
             jp    nz, +
             ld    a,(hl)
             bit   1, a
             jp    finTest
+:           cp    SOUTH
             jp    nz, +
             ld    a,(hl)
             bit   2, a
             jp    finTest
+:           ld    a,(hl)
             bit   3, a
finTest:     ret
.ends

.section "Animation tables" free
; cel array for shifting between legs apart (char $10) and wide ($11)
animWalk:
.db $10 $10 $10 $10 $11 $11 $11 $11 $ff
.ends

.section "Level 1 data: Village on fire (abbrev. 'fire')" free
; map width and height
fireMap:     .db   13 6
             .db   0 0 0 64 63

; this is the data for detecting collision with walls for each cell
fireCell:
             .db   $09 $01 $01 $01 $01 $05 $05 $01 $03 $00 $00 $00 $00
             .db   $08 $00 $00 $00 $02 $00 $00 $08 $00 $05 $05 $05 $07
             .db   $08 $00 $00 $04 $02 $00 $00 $0C $02 $00 $00 $00 $00
             .db   $0C $00 $02 $00 $0A $00 $00 $00 $0A $00 $00 $00 $00
             .db   $00 $0C $04 $05 $02 $00 $00 $09 $06 $00 $00 $00 $00
             .db   $00 $00 $00 $00 $0C $05 $05 $06 $00 $00 $00 $00 $00

; palette data for banks 1 and 2
firePal:     .db   BLACK SANDY MARINE BLACK BLACK BLACK BLACK BLACK
             .db   BLACK BLACK BLACK BLACK BLACK BLACK BLACK BLACK
             .db   BLACK YELLOW MAROON SIENNA GREY CRIMSON BLACK BLACK
             .db   BLACK BLACK BLACK BLACK BLACK BLACK BLACK BLACK

; tiles to be loaded for level 1, incl. sprites and background
fireTile:
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 1
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 2
             .db   $00 $ff $00 $00
             .db   $00 $ff $00 $00
             .db   $00 $ff $00 $00
             .db   $00 $ff $00 $00
             .db   $00 $ff $00 $00
             .db   $00 $ff $00 $00
             .db   $00 $ff $00 $00
             .db   $00 $ff $00 $00
; = tile 3
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 4
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 5
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 6
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 7
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 8
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 9
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 10
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 11
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 12
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 13
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 14
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 15
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
             .db   $ff $00 $00 $00
; = tile 16 - Arthur standing, w. legs apart
             .db   $00 $00 $01 $00
             .db   $00 $3c $01 $00
             .db   $28 $00 $15 $00
             .db   $3c $00 $01 $00
             .db   $7e $7e $01 $00
             .db   $7d $08 $14 $00
             .db   $3c $00 $3c $00
             .db   $00 $24 $00 $00
; = tile 17 -  Arthur w. legs together
             .db   $00 $00 $01 $00
             .db   $00 $3c $01 $00
             .db   $28 $00 $15 $00
             .db   $3c $00 $01 $00
             .db   $7e $7e $01 $00
             .db   $7d $08 $14 $00
             .db   $3c $00 $3c $00
             .db   $00 $18 $00 $00

fireScrn:
.include "fireScrn.inc"
.ends
